<?php
/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * Distributed under the GPL - see LICENSE
 *
 */


/**
 * @purpose represent a category within the system
 * @version $Revision: 296 $
 * $Id: category.php 502 2006-08-15 08:37:40Z kevbob $
 */

require_once 'iterator.php';


define('FOF_DEFAULT_CATEGORY', 'None');


class FOF_CategoryModel extends FOF_BaseModel
{
   function create()
   {
      return new FOF_Category();
   }
   
   
   function install()
   {
   	require_once 'ddl/category.php';
      
      $ddl_helper = new FOF_CategoryDDLHelper($this);
      return $ddl_helper->getCommands(); 
   }
   
   
   /**
    * retrieve a category instance by id
    * 
    */
   function fetch( $id )
   {
      $db = $this->getDB();
      
      $row = $db->getRow("SELECT *" .
                         "  FROM ".FOF_CATEGORY_TABLE.
                         " WHERE id = ?", array((int)$id));
      if ($row != false ) {
         return $this->row2instance( $row );
      }
      else {
         return false;
      }
   }
   
   /**
    * return default category instance
    * 
    * @param void
    * @return FOF_Category
    */
   function fetchDefaultCategory()
   {
      $category = $this->findCategory(FOF_DEFAULT_CATEGORY);

      if (!$category) {
         $category = $this->create();
         $category->setTitle(FOF_DEFAULT_CATEGORY);
      }
      return $category;
   }
   
   
   /**
    * search for a category instance by title
    * 
    * @param string $title
    * @return FOF_Category or false
    */
   function findCategory( $title )
   {
      $db = $this->getDB();
      
      $row = $db->GetRow("SELECT *" .
                         "  FROM ".FOF_CATEGORY_TABLE.
                         " WHERE title = ?", array($title) );
        
      if ($row != false ) {
         return $this->row2instance( $row );
      }
      else {
         return false;
      }
      
   }
   
   /**
    * persist any changes to a category to the database
    * if it does not exist in the db, create a row for it
    * 
    * @param FOF_Category $category
    */
   function save( &$category )
   {
      $db = $this->getDB();
      
      if ( $category->getId() == -1 ) {
         // insert category
         $sql = "INSERT INTO ".FOF_CATEGORY_TABLE.
                " (title,description)" .
                " VALUES(?,?)";
                
         $retval = $db->Execute($sql, array($category->getTitle(),
                                            $category->getDescription()));
         
         if (!$retval) {
         	return $retval;
         }
         
         $new_id = $db->Insert_ID();
                  
         $category->setId( $new_id );
      }
      else {
         // update user
         $sql = "UPDATE ".FOF_CATEGORY_TABLE.
                " SET title = ?," .
                "     description = ?" .
                " WHERE id = ?";

         $updateArgs = array($category->getTitle(),
                             $category->getDescription(),
                             (int)$category->getId());
         $retval = $db->Execute($sql, $updateArgs);
      }
      return $retval;
   }

   
   /**
    * remove category
    * 
    * @param FOF_Category $category
    */
   function remove( &$category )
   {
   	
      if ($category->getTitle() == FOF_DEFAULT_CATEGORY) {
      	return false;
      }
   
      $db = $this->getDB();

      // search for and reset category on any feeds in this
      // category
      $feed_model = FOF_ModelFactory::getFeedModel();
      $feed_search = $feed_model->getDefaultSearch();
      $feed_search->setCategory($category->getId());
      
      $feeds = $feed_model->search($feed_search);
      $feeds->rewind();
      
      $def_category = $this->fetchDefaultCategory();
      while ($feeds->valid()) {
      	$feed = $feeds->current();
         $feeds->next();
         
         // change category to default category and save
         $feed->setCategory($def_category);
         $feed_model->save($feed);
         
      }
            
      $sql = "DELETE FROM ".FOF_CATEGORY_TABLE.
             " WHERE id = ?";
      
      $retval = $db->Execute($sql, array((int)$category->getId()));
      
      $category->setId(-1);
      return $retval;
   }
   
   /**
    * retrieve sequence of all categories in the db
    * 
    * @param void
    * @return FOF_Iterator
    */
   function getCategories()
   {
      $db = $this->getDB();
      
      $result = $db->Execute("SELECT id, title, description" .
                              "  FROM ".FOF_CATEGORY_TABLE.
                              " ORDER BY title");
      
      if ($result != false) {
         $result_it = new FOF_ModelIterator($result, $this);
      }
      else {
         $result_it = new FOF_EmptyIterator();   
      }
      return $result_it;
      
   }
   
   /**
    * get count of feeds in category
    * 
    * @param FOF_Category
    * @return int
    */
   function getFeedCount( $category )
   {
      $db = $this->getDB();
      
      $result = $db->getOne("SELECT count(*)" .
                            "  FROM ".FOF_FEED_TABLE.
                            " WHERE category_id = ?",
                            array((int)$category->getId()));
      if (!$result) {
         return 0;
      }
      else {
         return $result;
      }
   }
   
   /**
    * 
    * @param array $row
    * @return FOF_Category
    */
   function row2instance( $row )
   {
      $cat = new FOF_Category( $row['id'] );
      $cat->setTitle( $row['title'] );
      $cat->setDescription( $row['description'] );
      return $cat;
   }
}


class FOF_Category
{
   var $id;
   var $title;
   var $description;
   
   function FOF_Category( $id = -1 )
   {
      $this->id = $id;
   }
   
   function getId()
   {
      return $this->id;
   }
   
   function setId( $id )
   {
      $this->id = $id;
   }
   
   function getTitle()
   {
      return $this->title;
   }
   
   function setTitle( $title )
   {
      $this->title = $title;
   }
   
   function getDescription()
   {
      return $this->description;
   }
   
   function setDescription( $desc )
   {
      $this->description = $desc;
   }
}
