<?php
/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * Distributed under the GPL - see LICENSE
 *
 */


/**
 * @purpose provide object and access/manipulation methods for feeds
 * @version $Revision: 296 $
 * $Id: feed.php 493 2006-08-09 13:08:12Z kevbob $
 */

require_once 'feed_search.php';
require_once 'category.php';
require_once 'item.php';
require_once 'iterator.php';


class FOF_FeedModel extends FOF_BaseModel
{
   var $itemCountCache = array();
   
   // retrieval methods
   
    /**
    * installer for this model
    * 
    * @param void
    * @return array
    */
   function install()
   {
      require_once 'ddl/feed.php';
      $ddl_helper = new FOF_FeedDDLHelper($this);
      return $ddl_helper->getCommands();
   }
   
   
   /**
    * return new feed object
    * 
    * @param void
    * @return FOF_Feed
    */
   function create()
   {
      return new FOF_Feed();
   }
   
   /**
    * fetch a single feed instance using the id
    * 
    * @param int $id
    */
   function fetch( $id )
   {
      $db = $this->getDB();
      // query database
      
      $row = $db->GetRow("SELECT feed.id, feed.url, feed.title," .
                         "       feed.link, feed.category_id," .
                         "       cat.title as category_title," .
                         "       feed.latitude, feed.longitude, feed.update_enabled," .
                         "       feed.auto_tag, feed.last_update, feed.update_interval" .
                         "  FROM ".FOF_FEED_TABLE." feed" .
                         "       JOIN ".FOF_CATEGORY_TABLE." cat ON (feed.category_id = cat.id)".
                         " WHERE feed.id = ?", array((int)$id));
      
      return $this->row2instance($row);
   }
   
   /**
    * fetch a single feed from db using url
    * 
    * @param string $url
    * @return FOF_Feed or false
    */
   function findFeed( $url )
   {
      $db = $this->getDB();
      // query database
      
      $row = $db->GetRow("SELECT feed.id, feed.url, feed.title," .
                         "       feed.link, feed.category_id," .
                         "       cat.title as category_title," .
                         "       feed.latitude, feed.longitude, feed.update_enabled," .
                         "       feed.auto_tag, feed.last_update, feed.update_interval" .
                         "  FROM ".FOF_FEED_TABLE." feed" .
                         "  JOIN ".FOF_CATEGORY_TABLE." cat ON (feed.category_id = cat.id)".
                         " WHERE feed.url = ?", array($url));
      
      return $this->row2instance($row);
   }

   /**
    * perform a search for feeds
    * this should be used anytime a client needs a list
    * of feeds from the DB
    * 
    * @param FOF_FeedSearch $search
    * @return FOF_FeedIterator
    */
   function search( $search )
   {

      $db = $this->getDB();
      $args = array();
      $sql = "SELECT feed.id, feed.url, feed.title," .
                         "       feed.link, feed.category_id," .
                         "       cat.title as category_title," .
                         "       feed.latitude, feed.longitude, feed.update_enabled," .
                         "       feed.auto_tag, feed.last_update, feed.update_interval" .
                         "  FROM ".FOF_FEED_TABLE." feed" .
                         "  JOIN ".FOF_CATEGORY_TABLE." cat ON (feed.category_id = cat.id) ";
      
      // add filter for category if value is not NULL
      if (!is_null($search->getCategory())) {
      	$cat_id = $search->getCategory();
         $sql .= "WHERE feed.category_id = ? ";
         $args[] = (int)$cat_id;
      }
      
        $sql .= sprintf( "ORDER BY %s %s", $search->getSortColumn(),
                                         $search->getSortDirection());
      
      $result = $db->Execute($sql, $args);

      if ($result != false) {
         $result_it = new FOF_ModelIterator($result, $this);
      }
      else {
         $result_it = new FOF_EmptyIterator();   
      }
      return $result_it;
   }
   
   /**
    * return default feed search object
    * this is a helper method that auto-populates a feed search
    * with default behavior from user or global options
    * 
    * @param void
    * @return FOF_FeedSearch
    */
   function getDefaultSearch()
   {
      return new FOF_FeedSearch();
   }

   /**
    * get total number of items in the feed
    * 
    * @param FOF_Feed $feed
    * @return int
    */
   function getTotalItemCount( $feed )
   {
   	

   	$feed_id = $feed->getId();
      if (!array_key_exists('totals', $this->itemCountCache)) {
      	$this->refreshTotalItemCounts();
      }
      
      if (isset($this->itemCountCache['totals'][$feed_id])) {
         return $this->itemCountCache['totals'][$feed_id];
      }
      else {
      	return 0;
      }
   }

   /**
    * get count of all items in db
    * 
    * @param void
    * @return int
    */
   function getTotalItems()
   {
      if (!array_key_exists('totals', $this->itemCountCache)) {
         $this->refreshTotalItemCounts();
      }
      
      $total_count = 0;
      foreach (array_keys($this->itemCountCache['totals']) as $feed_id ) {
      	$total_count += $this->itemCountCache['totals'][$feed_id];
      }
   	return $total_count;
   }
   
   /**
    * pre-fetch all total item counts
    * 
    * @param void
    * @return void
    */
   function refreshTotalItemCounts()
   {
   	$db = $this->getDB();
      
      $result = $db->Execute("SELECT feed_id, count(*) as cnt" .
                             " FROM ".FOF_ITEM_TABLE.
                             " GROUP BY feed_id");
      
      $totalCounts = array();

      while ($row = $result->FetchRow() ) {
         $totalCounts[$row['feed_id']] = $row['cnt'];
      }
      $result->Close();
      $this->itemCountCache['totals'] = $totalCounts;
   }

   /**
    * get item count for a particular item flag
    * 
    * @param FOF_Feed $feed
    * @param int $flag (one of flag constants defined in item model )
    */
   function getFlagItemCount( $feed, $flag )
   {
   	$feed_id = $feed->getId();
      $flag_key = sprintf("flag%d", $flag);
      
      if (!array_key_exists($flag_key, $this->itemCountCache)) {
         $this->refreshFlagItemCounts($flag);
      }
      
      if (isset($this->itemCountCache[$flag_key][$feed_id])) {
      	return $this->itemCountCache[$flag_key][$feed_id];
      }
      else {
      	return 0;
      }
   }

   /**
    * get count of all items in db
    * 
    * @param void
    * @return int
    */
   function getTotalFlagItems( $flag)
   {

      $flag_key = sprintf("flag%d", $flag);

   	
      if (!isset($this->itemCountCache[$flag_key])) {
         $this->refreshFlagItemCounts($flag);
      }
      
      $flag_count = 0;
      foreach (array_keys($this->itemCountCache[$flag_key]) as $feed_id ) {
         $flag_count += $this->itemCountCache[$flag_key][$feed_id];
      }
      return $flag_count;
   }


   /**
    * pre-fetch all counts for items with a given flag
    * 
    * @param void
    * @return void
    */
   function refreshFlagItemCounts( $flag )
   {
      $db = $this->getDB();
      
      $result = $db->Execute("SELECT feed_id, count(*) as cnt" .
                             " FROM ".FOF_ITEM_TABLE.
                             " WHERE flag = ?".
                             " GROUP BY feed_id", array($flag));
      
      $flagCounts = array();
      while ($row = $result->FetchRow() ) {
      	$flagCounts[$row['feed_id']] = $row['cnt'];
      }
      $result->Close();
      
      $flag_key = sprintf("flag%d", $flag);
      $this->itemCountCache[$flag_key] = $flagCounts;
   }


   /**
    * delete a feed from the database
    * 
    * @param FOF_Feed $feed
    */
   function remove( &$feed ){

      // get list of items for feed
      $item_model = FOF_ModelFactory::getItemModel();
      
      $item_search = new FOF_ItemSearch();
      $item_search->setFeed( $feed->getId());
      $item_search->setLimit(0);
      $item_search->setOffset(0);
      $item_search->setSortField('i.id');
      
      $item_iterator = $item_model->search($item_search);
      $item_iterator->rewind();

      // for each item, delete item
      while ($item_iterator->valid()) {
         $item = $item_iterator->current();
         $item_iterator->next();
         
         $val = $item_model->remove($item);
         if (!$val) {
            return $val;
         }
      }
      
      // delete feed
      $db = $this->getDB();
      
      $retval = $db->Execute("DELETE FROM ".FOF_FEED_TABLE.
                             " WHERE id = ?", array($feed->getId()));
      
      // reset $feed so it is a transient instance (no longer tied to a row in the db)
      $feed->setId( -1 );
      return $retval;
   }
   
   /**
    * save a feed to the database. detect which method should be executed
    * 
    * @param FOF_Feed $feed
    * @return boolean
    */
   function save( &$feed )
   {
      $db = $this->getDB();
      
      if ( $feed->getId() == -1 ) {
         
          $sql = "insert into ".FOF_FEED_TABLE.
                "        (url,title,link,category_id, auto_tag, update_enabled)" .
                "       values (?, ?, ?, ?, ?, ?)";
         $category = $feed->getCategory();
         $autotag_int = ($feed->isAutoTag()) ? 1 : 0;
         $update_en_int = ($feed->isUpdateEnabled()) ? 1 : 0;
         
         $retval = $db->Execute($sql, array(
                                        $feed->getUrl(),
                                        $feed->getTitle(),
                                        $feed->getLink(),
                                        (int)$category->getId(),
                                        $autotag_int,
                                        $update_en_int));
         
      }
      else {
      	
         $last_update_string = ($feed->getLastRssUpdate()) ? $db->DBTimeStamp($feed->getLastRssUpdate()) : 'NULL';
         $sql = "UPDATE ".FOF_FEED_TABLE.
                 "    SET url=?, title=?, link=?," .
                 "        category_id=?," .
                 "       latitude=?, longitude=?," .
                 "       auto_tag=?," .
                 "       update_enabled=?," .
                 "       update_interval=?," .
                 "       last_update=$last_update_string" .
                 " WHERE id = ?";
         $cat = $feed->getCategory();
         $queryInput = array( $feed->getUrl(),
                              $feed->getTitle(),
                              $feed->getLink(),
                              (int)$cat->getId(),
                              $feed->getLatitude(),
                              $feed->getLongitude(),
                              (int)$feed->isAutoTag(),
                              (int)$feed->isUpdateEnabled(),
                              $feed->getUpdateInterval(),
                              (int)$feed->getId());
         $retval = $db->Execute($sql, $queryInput);
      }
      return $retval;
   }

   /**
    * convert db result row to an FOF_Feed instance
    * 
    * @param array $row
    * @param boolean $extended
    * @return FOF_Feed
    */
   function row2instance( $row, $extended = true )
   {
      
      if (!$row) {
         return false;
      }
      
      $db = $this->getDB();
      
      // the FOF_FeedExt provides access back to the model in order to
      // retrieve values for derived or non-db fields
      if ($extended) {
         $feed = new FOF_FeedExt( $row['id'], $this);
      }
      else {
         $feed = new FOF_Feed( $row['id']);
      }
      
      $feed->setUrl($row['url']);
      $feed->setTitle($row['title']);
      $feed->setLink($row['link']);

      $cat = new FOF_Category( $row['category_id']);
      $cat->setTitle($row['category_title']);
      $feed->setCategory( $cat );

      $feed->setLatitude($row['latitude']);
      $feed->setLongitude($row['longitude']);
      $feed->setAutoTag($this->_int2bool($row['auto_tag']));
      $feed->setUpdateEnabled($this->_int2bool($row['update_enabled']));
      
      // last_update is a poorly named column.  it should be changed
      // to something better
      $feed->setLastRssUpdate($db->UnixTimeStamp($row['last_update']));

      $feed->setUpdateInterval($row['update_interval']);

      return $feed;
   }
}


/**
 * data transfer class for Feeds
 *
 */
class FOF_Feed
{
   var $id;
   var $url;
   var $title;
   var $link;
   var $category;
   var $latitude;
   var $longitude;
   var $auto_tag;
   var $update_enabled;
   var $last_rss_update;
   var $update_interval;
   

   function FOF_Feed( $id = -1 )
   {
      $this->id = $id;
   }
   
   function setId( $id )
   {
      $this->id = $id;
   }

   function getId()
   {
      return $this->id;
   }
   
   function setUrl( $url )
   {
      $this->url = $url;
   }
   
   function getUrl()
   {
      return $this->url;
   }
   
   function setTitle( $title )
   {
      $this->title = $title;   
   }
   
   function getTitle()
   {
      return $this->title;
   }
   
   function setLink( $link )
   {
      $this->link = $link;
   }
   
   function getLink()
   {
      return $this->link;
   }
   
   function setCategory( $category )
   {
      $this->category = $category;
   }
   
   function getCategory()
   {
      return $this->category;
   }
   
   function setLatitude( $latitude )
   {
      $this->latitude = $latitude;
   }
   
   function getLatitude()
   {
      return $this->latitude;
   }
   function setLongitude( $longitude )
   {
      $this->longitude = $longitude;
   }
   
   function getLongitude()
   {
      return $this->longitude;
   }
   
   function setAutoTag( $auto_tag )
   {
      $this->auto_tag = $auto_tag;
   }
   
   function isAutoTag()
   {
      return $this->auto_tag;
   }
   
   function setUpdateEnabled( $update_enabled )
   {
      $this->update_enabled = $update_enabled;
   }
      
   function isUpdateEnabled()
   {
      return $this->update_enabled;
   }
   
   function setLastRssUpdate( $last_update_ts )
   {
      $this->last_rss_update = $last_update_ts;
   }
      
   function getLastRssUpdate()
   {
      return $this->last_rss_update;
   }
   
   function getUpdateInterval()
   {
   	return $this->update_interval;
   }
   
   function setUpdateInterval( $interval_sec )
   {
   	$this->update_interval = $interval_sec;
   }
}


/**
 * 
 * 
 */
class FOF_FeedExt extends FOF_Feed
{
   var $_helper;
   var $model;
   
   function FOF_FeedExt( $id, &$model )
   {
      parent::FOF_Feed( $id );
      
      // $model *must* be passed and assigned by reference or the get*ItemCount()
      // methods on the model do not work correctly
      $this->model =& $model;
   }
   
   // extended fields are read-only.  Provide just the getters for them
   
   function getUnreadItemCount()
   {
      return $this->model->getFlagItemCount($this, FOF_UNREAD_FLAG);
   }
   
   function getTotalItemCount()
   {
      return $this->model->getTotalItemCount($this);
   }
   
   function getRssVersion()
   {
      $rss = FOF_ModelFactory::getRssModel();
      $rss_feed = $rss->getRssFeed($this->getUrl());
      if ($rss_feed) {
         return $rss_feed->getType();
      }
   }
   
   function getDescription()
   {
      $rss = FOF_ModelFactory::getRssModel();
      $rss_feed = $rss->getRssFeed($this->getUrl());
      if ($rss_feed) {
         return $rss_feed->getDescription();
      }
   }
   
   function getRssImage()
   {
      $rss = FOF_ModelFactory::getRssModel();
      $rss_feed = $rss->getRssFeed($this->getUrl());
      if ($rss_feed) {
         return $rss_feed->getImage();
      }
   }
}

