<?php
/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * Distributed under the GPL - see LICENSE
 *
 */


/**
 * @purpose represent an item search/filter within the system
 * @version $Revision: 296 $
 * $Id: item_search.php 519 2006-09-07 08:05:33Z kevbob $
 */


class FOF_ItemSearch
{

   var $filter;
   var $sortField;
   var $sortOrder;
   var $limit;
   var $offset;
   var $curDate;
	
	function FOF_ItemSearch()
	{
		// initialize defaults
      $this->filter = new FOF_AndCompositeFilter('root');
      $this->sortField = null;
      $this->sortOrder = null;
      $this->limit = 50;
      $this->offset = 0;
	}

   /**
    * allow performing deep copies of this object
    */
   function __clone()
   {
   	$this->filter = clone( $this->filter );
   }

   /**
    * expose addFilter() method of the underlying composite filter
    * 
    * @param FOF_SearchFilter $filter
    * @return void
    */
	function addFilter( $filter )
	{
		$this->filter->addFilter( $filter );
	}
   
   // filter convenience methods
   
   /**
    * set/update a filter for feed_id
    * 
    * @param int $feed_id
    */
   function setFeed( $feed_id )
   {
      $this->clearFCT();
      $feed_filt = new FOF_EqualFilter( 'feed_id', $feed_id );
      $this->filter->updateFilter($feed_filt);
   }
   
   /**
    * set/update a filter for category_id
    * 
    * @param int $category_id
    */
   function setCategory( $category_id )
   {
      $this->clearFCT();
      $cat_filt = new FOF_EqualFilter('category_id', $category_id);
      $this->filter->updateFilter($cat_filt);
      
   }
    
   function setTag( $tag_id )
   {
      $this->clearFCT();
      $tag_filt = new FOF_EqualFilter('tag_id', $tag_id);
      $this->filter->updateFilter( $tag_filt);
   }
   
   /**
    * remove any filters on feed, category, or tag
    * 
    * @param void
    * @return void
    */
   function clearFCT()
   {
      $this->filter->removeFilter('feed_id');
      $this->filter->removeFilter('category_id');
      $this->filter->removeFilter('tag_id');
   }
   
   function setFlag( $flag_id )
   {
      $flag_filt = new FOF_EqualFilter('i.flag', $flag_id);
      $this->filter->updateFilter($flag_filt);
   }
   
   /**
    * remove a flag filter if it exists
    */
   function unsetFlag()
   {
      $this->filter->removeFilter('i.flag');  
   }

   /**
    * set a substring search filter
    * 
    * @param string $search_term
    * @return void
    */
   function setSearch( $search_term )
   {
      $search_pattern = '%'.$search_term.'%';
      $search_filt = new FOF_OrCompositeFilter('search');
      $search_filt->addFilter(new FOF_IPatternFilter('i.title', $search_pattern) );
      $search_filt->addFilter(new FOF_IPatternFilter('i.content', $search_pattern) );
      $this->filter->updateFilter($search_filt);
   }
   
   /**
    * 
    * @param string $date (format Y/m/d)
    */
   function setDate( $date )
   {
      global $opts;

      // store the current value unchanged
      // this is used by the prev/next pager methods below
      $this->curDate = $date;

      // convert date to unix epoch timestamp
      $begin_ts = strtotime($date);
      // adjust to local TZ
      $begin_ts = $begin_ts + ($opts->get('global.time_offset') * 60 * 60);
      $end_ts = $begin_ts + (24 * 60 * 60);

      $dateFilter = new FOF_RangeFilter('publish_time', $begin_ts, $end_ts);

      // set filter value formatter that will convert the values from
      // unix timestamps to 
      $dateFilter->setFormatter(new FOF_DBTimeFormatter());
      $this->filter->updateFilter($dateFilter);
   }

   // end filter convenience methods

   // begin paging convenience functions

   function getNextDate()
   {
      global $opts;
      if (!isset($this->curDate)) {
         die('i must have a date set with setDate() first');
      }
      
      $datets = strtotime($this->curDate);
      // bump timestamp up a day
      $datets = $datets + (24 * 60 * 60);
      $newDate = date( "Y/m/d", $datets - ($opts->get('global.time_offset') * 60 * 60) );

      $newSearch = clone($this);
      $newSearch->setDate($newDate);
      return $newSearch;
   }
   
   function getPrevDate()
   {
      global $opts;
      if (!isset($this->curDate)) {
         die('i must have a date set with setDate() first');
      }
      
      $datets = strtotime($this->curDate);
      // bump timestamp down a day
      $datets = $datets - (24 * 60 * 60);
      $newDate = date( "Y/m/d", $datets - ($opts->get('global.time_offset') * 60 * 60) );

      // clone the current search object and set the new date filter
      $newSearch = clone($this);
      $newSearch->setDate($newDate);
      return $newSearch;
   }
   
   
   /**
    * retrieve search object with what is the next higher offset value
    * 
    * @param void
    * @return FOF_ItemSearch
    */
   function getNextOffset()
   {
      $new_offset = $this->offset + $this->limit;
      $new_search = clone($this);
      $new_search->setOffset($new_offset);
      return $new_search;
   
   }
   
   /**
    * retrieve search object with what is the previous (lower) offset value
    * 
    * @param void
    * @return FOF_ItemSearch
    */
   function getPrevOffset()
   {
      $new_search = clone($this);
      $new_offset = $this->offset - $this->limit;
      
      if ($this->offset > 0
          and $new_offset < 0 ) {
         $new_offset = 0;    	
      }
      
      $new_search->setOffset($new_offset);

      return $new_search;
   }

   // end paging convenience functions
	
   function setSortField( $field )
   {
      $this->sortField = $field;	
   }
	
	function setSortOrder( $order )
	{
		$this->sortOrder = $order;		
	}
	
   function setLimit( $limit )
   {
      $this->limit = $limit;
   }
   
   function setOffset( $offset )
   {
      $this->offset = $offset;
   }
   
	# define get functions
	
   function getFilter()
   {
      return $this->filter;
   }
   
	function getSortField()
	{
		return $this->sortField;
	}
	
	function getSortOrder()
	{
		return $this->sortOrder;
	}
   
   function getLimit()
   {
      return $this->limit;
   }
   
   function getOffset()
   {
      return $this->offset;
   }
}


# this is the interface all filter classes must implement.
//interface FOF_SearchFilter
//{
//
//   /**
//   * 
//   * @param void
//   * @return array (<sql segment>, <array of arguments>)
//   */
//   function getSQL();
//}

class FOF_EqualFilter /* implements FOF_SearchFilter */
{

   function FOF_EqualFilter( $column, $value)
   {
      $this->column = $column;
      $this->value  = $value;
   }

   function getColumn()
   {
      return $this->column;
   }

   function getValue()
   {
      return $this->value;
   }
   
   function getSQL()
   {
      $sqlArgs = array();
      if (is_null($this->value)) {
         $sqlString = sprintf( "%s IS NULL", $this->column);
      }
      else {
         $sqlString = sprintf( "%s = ?", $this->column );
         array_push($sqlArgs, $this->value);        
      }
      return array($sqlString, $sqlArgs);
   }
}


class FOF_NotEqualFilter /* implements FOF_SearchFilter */
{
   var $column;
   var $value;
   function FOF_NotEqualFilter( $column, $value)
   {
      $this->column = $column;
      $this->value  = $value;
   }
   
   function getColumn()
   {
      return $this->column;
   }

   function getValue()
   {
      return $this->value;
   }

   function getSQL()
   {
      $sqlArgs = array();
      if (is_null($this->value)) {
         $sqlString = sprintf( "%s IS NOT NULL", $this->column);
      }
      else {
         $sqlString = sprintf( "%s != ?", $this->column );
         array_push($sqlArgs, $this->value); 
      }
      return array($sqlString, $sqlArgs);
   }
}


class FOF_LessThanFilter /* implements FOF_SearchFilter */
{
   var $column;
   var $value;
   var $formatter;
   
   function FOF_LessThanFilter( $column, $value)
   {
      $this->column = $column;
      $this->value  = $value;
   }
   
   function getColumn()
   {
      return $this->column;
   }

   function getValue()
   {
      return $this->value;
   }

   function setFormatter( $formatter )
   {
      $this->formatter = $formatter;
   }

   function getSQL()
   {
      if (isset($this->formatter)) {
         $sqlArgs = array($this->formatter->format($this->value));
      }
      else {
         $sqlArgs = array();
      }
      $sqlString = sprintf( "%s < ?", $this->column );

      return array($sqlString, $sqlArgs);
   }
   
}

class FOF_GreaterThanFilter /* implements FOF_SearchFilter */
{
   var $column;
   var $value;
   var $formatter;
   
   function FOF_GreaterThanFilter( $column, $value)
   {
      $this->column = $column;
      $this->value  = $value;
   }
   
   function getColumn()
   {
      return $this->column;
   }

   function getValue()
   {
      return $this->value;
   }

   function setFormatter( $formatter )
   {
      $this->formatter = $formatter;
   }
   
   function getSQL()
   {
      if (isset($this->formatter)) {
         $sqlArgs = array($this->formatter->format($this->value));
      }
      else {
         $sqlArgs = array();
      }
      $sqlString = sprintf( "%s > ?", $this->column );

      return array($sqlString, $sqlArgs);
   }
   
}


class FOF_RangeFilter /* implements FOF_SearchFilter */
{
   var $column;
   var $lowValue;
   var $highValue;
   var $formatter;
   
   function FOF_RangeFilter( $column, $lowValue, $highValue )
   {
      $this->column = $column;
      $this->lowValue = $lowValue;
      $this->highValue = $highValue;
   }

   function getColumn()
   {
      return $this->column;
   }

   function getLowValue()
   {
   	return $this->lowValue;
   }

   function setFormatter( $formatter )
   {
      $this->formatter = $formatter;
   }
   
   function getSQL()
   {
      if (isset($this->formatter)) {
         $sqlArgs = array( $this->formatter->format($this->lowValue),
                          $this->formatter->format($this->highValue) );
      }
      else {
         $sqlArgs = array($this->lowValue, $this->highValue);
      }
      $sqlString = sprintf( '%s BETWEEN ? AND ?', $this->column );
      return array($sqlString, $sqlArgs);
   }
}


class FOF_InFilter /* implements FOF_SearchFilter */
{
   var $column;
   var $values;
   var $formatter;
   
   function FOF_InFilter( $column, $values )
   {
      $this->column = $column;
      $this->values = $values;
   }

   function getColumn()
   {
      return $this->column;
   }

   function setFormatter( $formatter )
   {
      $this->formatter = $formatter;
   }
   
   function getSQL()
   {
      if (isset($this->formatter)) {
         $sqlArgs = array();
         
         foreach ($this->values as $val) {
            array_push($sqlArgs, $this->formatter->format($val));
         }
      }
      else {
         $sqlArgs = $this->values;
      }
      
      $qmarks = array();
      for ($i = 0; $i < count($this->values); $i++) {
         $qmarks[] = '?';
      }

      $qmarks_string = join(', ', $qmarks);

      $sqlString = sprintf( '%s IN ( %s )', $this->column, join(', ', $qmarks) );

      return array($sqlString, $sqlArgs);
   }
}


class FOF_NotInFilter /* implements FOF_SearchFilter */
{
   var $column;
   var $values;
   var $formatter;
   
   function FOF_NotInFilter( $column, $values )
   {
      $this->column = $column;
      $this->values = $values;
   }

   function getColumn()
   {
      return $this->column;
   }

   function setFormatter( $formatter )
   {
      $this->formatter = $formatter;
   }
   
   function getSQL()
   {
      if (isset($this->formatter)) {
         $sqlArgs = array();
         
         foreach ($this->values as $val) {
            array_push($sqlArgs, $this->formatter->format($val));
         }
      }
      else {
         $sqlArgs = $this->values;
      }
      
      $qmarks = array();
      for ($i = 0; $i < count($this->values); $i++) {
         $qmarks[] = '?';
      }

      $qmarks_string = join(', ', $qmarks);

      $sqlString = sprintf( '%s NOT IN ( %s )', $this->column, join(', ', $qmarks) );

      return array($sqlString, $sqlArgs);
   }
}


class FOF_PatternFilter /* implements FOF_SearchFilter */
{
   var $column;
   var $value;
   
   function FOF_PatternFilter( $column, $value)
   {
      $this->column = $column;
      $this->value  = $value;
   }

   function getColumn()
   {
      return $this->column;
   }
   
   function getValue()
   {
      return $this->value;
   }


   function getSQL()
   {
      $sqlString = sprintf( "%s LIKE ?", $this->column );
      return array($sqlString, array($this->value));
   }
}

/**
 * case insesitive cousin of FOF_PatternFilter
 */
class FOF_IPatternFilter extends FOF_PatternFilter
{
   function getSQL()
   {
      $lc_value = strtolower($this->value);
      $sqlString = sprintf( "LOWER(%s) LIKE ?", $this->column );
      return array($sqlString, array($lc_value));
   }
}


class FOF_NotPatternFilter /* implements FOF_SearchFilter */
{
   var $column;
   var $value;
   
   function FOF_NotPatternFilter( $column, $value)
   {
      $this->column = $column;
      $this->value  = $value;
   }

   function getColumn()
   {
      return $this->column;
   }

   function getValue()
   {
      return $this->value;
   }

   function getSQL()
   {
      $sqlString = sprintf( "%s NOT LIKE ?", $this->column );
      return array($sqlString, array($this->value));
   }
}


class FOF_CompositeFilter /* implements FOF_SearchFilter */
{
   var $filters;
   var $column;
   var $delimiter = "";
   
   function FOF_CompositeFilter($column)
   {
      $this->column = $column;
      $this->filters = array();
   }

   /**
    * allow for making deep copies of this object
    */
   function __clone()
   {
   	
      $filters = array();
      
      foreach ($this->filters as $filt) {
      	$filters[] = clone( $filt );
      }
      $this->filters = $filters;

   }

   function addFilter( $filter )
   {
      $this->filters[] = $filter;
   }
   
   /**
    * add or update a filter in the filter list using column value
    * 
    * @param string $column
    * @param FOF_SearchFilter
    */
   function updateFilter( $filter )
   {
      $column = $filter->getColumn();
      $found = false;
      foreach (array_keys($this->filters) as $index) {
         if ($this->filters[$index]->getColumn() == $column) {
            $this->filters[$index] = $filter;
            $found = true;
            break;
         }
      }
      // if there was not a suitable filter to replace with this filter,
      // add this one
      if (!$found) {
         $this->addFilter($filter);
      }
   }
   
   /**
    * remove a filter from the filter list using column value
    */
   function removeFilter( $column )
   {
      foreach (array_keys($this->filters) as $index) {
         if ($this->filters[$index]->getColumn() == $column) {
            unset($this->filters[$index]);
         }
      }
   }
   
   /**
    * locate a filter by column name
    */
   function findFilter( $column )
   {
      foreach ($this->filters as $filter) {
         if ($filter->getColumn() == $column) {
            return $filter;
         }
         // if we contain a composite filter, search inside them also
         elseif (is_a($filter, 'FOF_AndCompositeFilter')) {
            $val = $filter->findFilter();
            if ($val != false) {
               return $val;
            } 
         }
      }
      return false;
   }
   

   /**
    * return value for column field
    **/
   function getColumn()
   {
      return $this->column;
   }
   
   function getSQL()
   {
      if ($this->delimiter === "") {
         die("must subclass and specify delimiter field");
      }
      
      // because we are a composite class, we must
      // walk all filters we encapsulate and collect their 
      $output = "";
      $sqlArgs = array();
      
      foreach ($this->filters as $filter) {
         if (!$filter) { continue; }
         list($fSQL, $fArgs) = $filter->getSQL();
         if ($output === "") {
            $output .= "$fSQL\n";
         }
         else {
            $output .= " ".$this->delimiter." " . $fSQL . "\n";
         }
         $sqlArgs = array_merge($sqlArgs, $fArgs);
      }
      if (count($this->filters) == 0) {
         $output = "1 = 1";
      }

      return array(sprintf( "( %s )", $output), $sqlArgs);
   }
}


class FOF_AndCompositeFilter extends FOF_CompositeFilter
{
   var $delimiter = 'AND';
}

class FOF_OrCompositeFilter extends FOF_CompositeFilter
{
   var $delimiter = "OR";
}

/**
 * 
 * TODO rewrite this to not use global DB connection
 */
class FOF_DBTimeFormatter
{
   function format( $val )
   {
      $db = FOF_BaseModel::getDB();
      $timeVal = $db->DBTimeStamp($val);
      // now strip quotes off of the timeVal
      return str_replace("'", '', $timeVal);
      
   }
}

