<?php
/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * Distributed under the GPL - see LICENSE
 *
 */


/**
 * @purpose provide consistent interface to rss fetcher/parser
 * @version $Revision: 296 $
 * $Id: rss.php 521 2006-10-04 02:21:13Z kevbob $
 */

// set some constants used by magpie
define('MAGPIE_CACHE_AGE', 60*15);
define('MAGPIE_USER_AGENT', sprintf('fofRedux/%s (+http://fofredux.sourceforge.net/)',FOF_VERSION) );
define('MAGPIE_CACHE_DIR', FOF_CACHE_DIR);
define('MAGPIE_DEBUG', null);


// import MagpieRSS files
require_once FOF_DIR . 'magpierss/rss_fetch.inc';

require_once FOF_DIR . 'lib/util/iterator.class.php';


class FOF_RssModel
{
   
   function FOF_RssModel()
   {
      $this->rss_cache = new RSSCache();
   }
   
   /**
    *
    * @param FOF_Feed $feed
    * @return int feed cache file age in seconds
    */
   function getFileAge( &$feed )
   {
      return $this->rss_cache->cache_age( $feed->getUrl() . MAGPIE_OUTPUT_ENCODING );
   }
   
   /**
    * 
    * @param string $url
    * @param boolean $useRemote update from source
    */
   function getRssFeed( $url, $useRemote = false )
   {
      if ( $useRemote ) {
         $rss =& fetch_rss( $url );
      }
      else {
         $rss =& $this->rss_cache->get( $url . MAGPIE_OUTPUT_ENCODING);
      }
      

      if (!isset($rss->channel)) {
         // some error happened.  Let the caller know
         return false;
      }
      
      // instaintiate rss feed object, and return it
      $rss_feed = new FOF_RssFeed( $rss, $url );
      return $rss_feed;

   }
   
   function error()
   {
      return magpie_error();
   }
}

/**
 * give a consistent abstraction 
 *  of the data in an rss feed xml file
 */
class FOF_RssFeed
{
   var $rss;
   var $url;
   
   function FOF_RssFeed( $rss, $url )
   {
      $this->rss = $rss;
      $this->url = $url;
   }
   
   // this object is read only. only getter methods are provided
   
   function getUrl()
   {
   	return $this->url;
   }
   
   function getLink()
   {
   	
      if (isset($this->rss->channel['link'])) {
      	return $this->rss->channel['link'];
      }
      else {
      	return null;
      }
   }
   
   function getTitle()
   {
      if (isset($this->rss->channel['title'])) {
         return $this->rss->channel['title'];
      }
      else {
         return null;
      }
   }
   
   function getType()
   {
      return $this->rss->feed_type;
   }
   
   function getVersion()
   {
      return $this->rss->feed_version;
   }
   
   
   function getDescription()
   {
      if (isset($this->rss->channel['description'])) {
         return $this->rss->channel['description'];
      }
      else {
         return null;
      }

   }
      
   function getImage()
   {
      if (isset($this->rss->image['url'])) {
         return $this->rss->image['url'];
      }
      else {
         return null;
      }
   }
   
   /**
    * 
    * @param void
    * @return 
    */
   function getItems()
   {
      return new FOF_RssItemIterator( $this );
   }
   
}

/**
 * abstraction for a normalized rss item
 * this compensates for the differences between rss 1.0, 2.0, 0.9, atom, joe bob syndication format Version 11eleven!1
 */
class FOF_RssItem
{
   var $item;
   var $feed_link;
   
   /**
    * @param array $item
    * @param array $feed
    */
   function FOF_RssItem( $item, $feed )
   {
      $this->item =& $item;
      $this->feed_link = $feed->getLink();
   }
   
   // this is a read-only class, only getter methods are provided
   
   function getLink()
   {
      if(isset($this->item['link'])) {
         $link = $this->item['link'];
      }
      elseif (isset($this->item['guid'])) {
         $link = $this->item['guid'];
      }
      elseif (isset($this->item['enclosure@url'])) {
         $link = $this->item['enclosure@url'];
      }
      else {
         $link = $this->feed_link;
      }
      
      return $link;
   }

   /**
    * returns unique cryptographic hash of the item.
    * this is used to differentiate unique items when other methods fail. (link or guid tag missing, etc)
    * 
    * @param void
    * @return string md5 hash of item
    */
   function getUniqueHash()
   {
   	
      // construct a string using the link, title, and content of the item
      
   	$inputString  = $this->getLink();
      $inputString .= $this->getTitle();
      
      // avoid some problems caused by slight changes in the content
      //$inputString .= $this->getContent();
      
      // hash the string and return it
      return sha1( $inputString );
   }


   function getTitle()
   {
      if(isset($this->item['title']))
      {
         $title = $this->item['title'];
      }
      else {
        $title = "[no title]";
      }
      return $title;
   }

   function getContent()
   {
      if (isset($this->item['content']['encoded'])) {
         $content = $this->item['content']['encoded'];
      }
      elseif (isset($this->item['atom_content'])) {
         $content = $this->item['atom_content'];
      }
      elseif (isset($this->item['description'])) {
         $content = $this->item['description'];
      }
      else {
         $content = null;
      }
      return $content;
   }
  
   function getPublishTime()
   {
      // determine publish timestamp for item
      $publish_ts = null;
      if (isset($this->item['issued']))
         $publish_ts = strtotime($this->item['issued']);
      if (isset($this->item['dc']['date']))
         $publish_ts = strtotime($this->item['dc']['date']);
      if (isset($this->item['pubdate']))
         $publish_ts = strtotime($this->item['pubdate']);
      
      // as a last resort, use current timestamp of server
      if ( is_null($publish_ts)
           or $publish_ts === -1
           or $publish_ts === false) {
          $publish_ts= time();
      }
      
      return $publish_ts;
   }
   
   function getCacheTime()
   {
      return time();
   }
   
   function getCreator()
   {
      if (isset($this->item['dc']['creator'])) {
         $dccreator = $this->item['dc']['creator'];
      } elseif (isset($this->item['author_name'])) {
         // pick up ATOM author name if available
         $dccreator = $this->item['author_name'];
      } else {
         $dccreator = null;
      }
      return $dccreator;
   }

   /**
    * 
    * @return string
    */
   function getEnclosureLink()
   {
      if (isset($this->item['enclosure@url'])) {
         $enclosure_link = $this->item['enclosure@url'];
      }
      else {
         $enclosure_link = null;
      }
      return $enclosure_link;
   }
   
   /**
    * 
    * @return string mime type of the enclosure url
    */
   function getEnclosureType()
   {
      if (isset($this->item['enclosure@type'])) {
         $enclosure_type = $this->item['enclosure@type'];
      }
      else {
         $enclosure_type = null;
      }
      return $enclosure_type;
   }

   function getLatitude()
   {
      if (isset($this->item['icbm']['latitude'])) {
         $latitude = $this->item['icbm']['latitude'];
      }
      elseif(isset($this->item['geo']['lat'])) {
         $latitude = $this->item['geo']['lat'];
      }
      elseif(isset($this->item['geo']['point']['point_lat'])) {
         $latitude = $this->item['geo']['point']['point_lat'];
      }
      elseif(isset($this->item['geourl']['latitude'])) {
         $latitude = $this->item['geourl']['latitude'];
      }
      elseif ( isset($this->item['georss']['point']) ) {
         list($latitude, $longitude) = split(" ",$this->item['georss']['point']);
      }
      else {
         $latitude = null;
      }
      return $latitude;
   }
   
   function getLongitude()
   {
      if (isset($this->item['icbm']['longitude'])) {
         $longitude = $this->item['icbm']['longitude'];
      }
      elseif(isset($this->item['geo']['long'])) {
         $longitude = $this->item['geo']['long'];
      }
      elseif(isset($this->item['geo']['point']['point_long']))
      {
         $longitude = $this->item['geo']['point']['point_long'];
      }
      elseif(isset($this->item['geourl']['longitude'])) {
         $longitude = $this->item['geourl']['longitude'];
      }
      elseif ( isset($this->item['georss']['point']) ) {
         list($latitude, $longitude) = split(" ",$this->item['georss']['point']);
      }
      else {
         $longitude = null;
      }
      return $longitude;      
   }
   
   /**
    * retrieve all tags for this item.
    * 
    * @return array
    */
   function getTags()
   {
      $itemTags = array();
         
      if ( isset($this->item['category#']) ) {
         // sometimes the first tag is overloaded with a comma separated list of categories
         if ( preg_match('/,/', $this->item['category']) ) {
            foreach (split(',', $this->item['category']) as $tag ) {
               $itemTags[] = trim($tag);
            }
         }
         else {
            $itemTags[] = $this->item['category'];
         }
            
         // for subsequent categories,  the names are slighly different
         if ( $this->item['category#'] > 1 ) {
            for ($i = 2; $i < ($this->item['category#'] + 1); $i++ ) {
               $itemTags[] = $this->item["category#${i}"];
            }
         }
      }
      elseif ( isset($this->item['dc']['subject#'])) {
         // sometimes the first tag is overloaded with a comma separated list of subjects
         if ( preg_match('/,/', $this->item['dc']['subject']) ) {
            $itemTags = split(',', $this->item['dc']['subject']);
         }
         else {
            $itemTags[] = $this->item['dc']['subject'];
         }
            
         // for subsequent categories,  the names are slighly different
         if ( $this->item['dc']['subject#'] > 1 ) {
            for ($i = 2; $i < ($this->item['dc']['subject#'] + 1); $i++ ) {
               $itemTags[] = $this->item['dc']["subject#${i}"];
            }
         }
            
            
         if ( $this->item['dc']['subject#'] > 1 ) {
         }
         else {
            $itemTags = split('/,/', $this->item['dc']['subject']);
         }
      }
      // catch flickr tags for item
      elseif ( isset($this->item['media']['category']) ) {
         $itemTags = split(' ', $this->item['media']['category']);
      }
      
      return $itemTags;
   }
}


class FOF_RssItemIterator extends FOF_ArrayIterator
{
   var $rss_feed;

   function FOF_RssItemIterator( $rss_feed )
   {
      $this->rss_feed = $rss_feed;
      // The iterator is a helper for the Rss feed.  it needs to read
      // the items array out of the magpie rss 
      parent::FOF_ArrayIterator( $rss_feed->rss->items );
   }


   /**
    * decorate parent method so we return an FOF_RssItem object
    * 
    */
   function _getNextItem()
   {
      $itemVal = parent::_getNextItem();

      if ( $itemVal != false ) {       
         return new FOF_RssItem( $itemVal, $this->rss_feed );
      }
      else {
         return false;
      }
   }
}



/**
 * scan through html page searching for rss links
 * 
 * @param string $html
 * @param string $location
 * @return string or bool
 */
function fof_getRSSLocation($html, $location){
    if(!$html or !$location){
        return false;
    }else{
        #search through the HTML, save all <link> tags
        # and store each link's attributes in an associative array
        $matches = null;
        preg_match_all('/<link\s+(.*?)\s*\/?>/si', $html, $matches);
        $links = $matches[1];
        $final_links = array();
        $link_count = count($links);
        for($n=0; $n<$link_count; $n++){
            $attributes = preg_split('/\s+/s', $links[$n]);
            foreach($attributes as $attribute){
                $att = preg_split('/\s*=\s*/s', $attribute, 2);
                if(isset($att[1])){
                    $att[1] = preg_replace('/([\'"]?)(.*)\1/', '$2', $att[1]);
                    $final_link[strtolower($att[0])] = $att[1];
                }
            }
            $final_links[$n] = $final_link;
        }
        #now figure out which one points to the RSS file
        for($n=0; $n<$link_count; $n++){
            if(strtolower($final_links[$n]['rel']) == 'alternate'){
                if(strtolower($final_links[$n]['type']) == 'application/rss+xml'){
                    $href = $final_links[$n]['href'];
                }
                if(!$href and strtolower($final_links[$n]['type']) == 'text/xml'){
                    #kludge to make the first version of this still work
                    $href = $final_links[$n]['href'];
                }
                if($href){
                    if(strstr($href, "http://") !== false){ #if it's absolute
                        $full_url = $href;
                    }else{ #otherwise, 'absolutize' it
                        $url_parts = parse_url($location);
                        #only made it work for http:// links. Any problem with this?
                        $full_url = "http://$url_parts[host]";
                        if(isset($url_parts['port'])){
                            $full_url .= ":$url_parts[port]";
                        }
                        if($href{0} != '/'){ #it's a relative link on the domain
                            $full_url .= dirname($url_parts['path']);
                            if(substr($full_url, -1) != '/'){
                                #if the last character isn't a '/', add it
                                $full_url .= '/';
                            }
                        }
                        $full_url .= $href;
                    }
                    return $full_url;
                }
            }
        }
        return false;
    }
}

