<?php
/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * Distributed under the GPL - see LICENSE
 *
 */


/**
 * @purpose represent a user within the system
 * @version $Revision: 296 $
 * $Id: user.php 493 2006-08-09 13:08:12Z kevbob $
 */

require_once 'iterator.php';


class FOF_UserModel extends FOF_BaseModel
{

   function create()
   {
      return new FOF_User();  
   }

   /**
    * installer for this model
    * 
    * @param void
    * @return array
    */
   function install()
   {
      require_once 'ddl/user.php';
      $ddl_helper = new FOF_UserDDLHelper($this);
      return $ddl_helper->getCommands();
   }

   /**
    * 
    */
   function fetch( $id )
   {
      $db = $this->getDB();
      
      $sql = "SELECT *" .
             "  FROM ".FOF_USER_TABLE.
             " WHERE id = ?";
      
      $row = $db->GetRow($sql, array((int)$id));
      
      if ($row != false) {
         return $this->row2instance($row);
      }
      else {
         return false;
      }
   }
   
   /**
    * save changes to a user
    * if it does not already exist, it is created and a new id is assigned
    * 
    * @param FOF_User $user
    * @return boolean
    */
   function save( &$user )
   {
      $db = $this->getDB();
      
      if ( $user->getId() == -1 ) {
         // insert user
         $sql = "INSERT INTO ".FOF_USERS_TABLE.
                " (username,password,email,admin,date_added,date_changed)" .
                " VALUES(?,?,?,?,{$db->sysTimeStamp},{$db->sysTimeStamp})";
                
         if ($user->isAdmin()) {
            $admin_int = 1;
         }
         else {
            $admin_int = 0;
         }
         $insertArgs = array($user->getUsername(),
                             $user->getPassword(),
                             $user->getEmail(),
                             $admin_int);
         $retval = $db->Execute($sql, $insertArgs);
         
         $new_id = $db->Insert_ID();
         
         // postgres's Insert_ID() method is broken. 
         // this is a hack to get around it but, the name
         // of the sequence may be predictable like below
         if (!$new_id) {
            $seqName = FOF_USERS_TABLE . '_id_seq';
            $new_id = $db->GetOne("select currval('$seqName')");
         }
         $user->setId( $new_id );
      }
      else {
         // update user
         $sql = "UPDATE ".FOF_USERS_TABLE.
                " SET username = ?," .
                "     password = ?," .
                "     email = ?," .
                "     admin = ?," .
                "     date_changed = {$db->sysTimeStamp}" .
                " WHERE id = ?";

         // admin flag in user object is boolean, convert to integer
         if ($user->isAdmin()) {
            $admin_int = 1;
         }
         else {
            $admin_int = 0;
         } 
         $updateArgs = array($user->getUsername(),
                             $user->getPassword(),
                             $user->getEmail(),
                             $admin_int,
                             (int)$user->getId());
         $retval = $db->Execute($sql, $updateArgs);
      }
      return $retval;
   }
   
   function remove( &$user )
   {
      $db = $this->getDB();
      
      $retval = $db->Execute("DELETE FROM ".FOF_USERS_TABLE.
                   " WHERE id = ?", array((int)$user->getId()));

      // the user object passed to us is now transient
      //meaning, it is not persisted in the model
      $user->setId( -1 );

      return $retval;
   }
   
   /**
    * givin username, retrieve the user from the database, or return false
    * 
    * @param string $username
    * @return FOF_User or false
    */
   function findUser($username)
   {
      $db = $this->getDB();
      
      $sql = "SELECT *" .
             "  FROM ".FOF_USERS_TABLE.
             " WHERE username = ?";
      
      $row = $db->GetRow($sql, array($username));
      
      if ($row != false) {
         return $this->row2instance($row);
      }
      else {
         return false;
      }
   }
   
   /**
    * retrieve sequence of all users in the db
    * 
    * @param void
    * @return FOF_UserIterator
    */
   function getUsers()
   {
      $db = $this->getDB();
      
      $result = $db->Execute("SELECT *" .
                              "  FROM ".FOF_USERS_TABLE.
                              " ORDER BY id");

      if ($result != false) {
         $users_it = new FOF_ModelIterator($result, $this);
      }
      else {
         $users_it = new FOF_EmptyIterator();
      }

      return $users_it;
   }
   
   /**
    * convert query row to a user object
    * 
    * @param array $row
    * @return FOF_User
    */
   function row2instance( $row )
   {
      $user = new FOF_User($row['id']);
      
      $user->setUsername( $row['username'] );
      $user->setPassword( $row['password'] );
      $user->setEmail( $row['email'] );
      $user->setAdmin( $row['admin'] );
      $user->setDateAdded( $row['date_added'] );
      $user->setDateChanged( $row['date_changed'] );
      
      return $user;
   }   
}


/**
 * class used to represent a user instance in the model
 * the primary use is as a DTO (data transfer object)
 * for passing data to/from the model
 * 
 * 
 */
class FOF_User
{
   // data members map to table columns.
   var $id;
   var $username;
   var $password;
   var $email;
   var $admin;
   var $date_added;
   var $date_changed;

   function FOF_User( $id = -1 )
   {
      $this->id = $id;  
   }

   function getId()
   {
      return $this->id;
   }
   
   function setId( $id )
   {
      $this->id = $id;
   }
   
   function getUsername()
   {
      return $this->username;
   }
   
   function setUsername( $username )
   {
      $this->username = $username;
   }
   
   function getPassword()
   {
      return $this->password;
   }
   
   function setPassword( $password )
   {
      $this->password = $password;
   }

   function getEmail()
   {
      return $this->email;
   }
   
   function setEmail( $email )
   {
      $this->email = $email;
   }

   function isAdmin()
   {
      return $this->admin;
   }
   
   function setAdmin( $admin )
   {
      if ($admin == 1) {
         $this->admin = true;
      }
      else {
         $this->admin = false;
      }
   }
   
   function getDateAdded()
   {
      return $this->date_added;
   }
   
   function setDateAdded( $datets )
   {
      $this->date_added = $datets;
   }
   
   function getDateChanged()
   {
      return $this->date_changed;
   }
   
   function setDateChanged( $datets )
   {
      $this->date_changed = $datets;
   }
}


